# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

from tvtk.tvtk_classes.data_reader import DataReader


class XdmfReader(DataReader):
    """
    XdmfReader - Reads e_xtensible Data Model and Formatfiles
    
    Superclass: DataReader
    
    XdmfReader reads XDMF data files so that they can be visualized
    using VTK. The output data produced by this reader depends on the
    number of grids in the data file. If the data file has a single
    domain with a single grid, then the output type is a DataSet
    subclass of the appropriate type, otherwise it's a
    MultiBlockDataSet.
    
    Refer to DataReader which provides many methods for controlling
    the reading of the data file.
    @warning
    Uses the XDMF API (http://www.xdmf.org)
    @sa
    DataReader
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkXdmfReader, obj, update, **traits)
    
    domain_name = traits.Trait(None, None, traits.String(enter_set=True, auto_set=False), help=\
        """
        Set the active domain. Only one domain can be selected at a time.
        By default the first domain in the datafile is chosen. Setting
        this to null results in the domain being automatically chosen.
        Note that if the domain name is changed, you should explicitly
        call update_information() before accessing information about
        grids, data arrays etc.
        """
    )

    def _domain_name_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetDomainName,
                        self.domain_name)

    def get_cell_array_status(self, *args):
        """
        V.get_cell_array_status(string) -> int
        C++: int GetCellArrayStatus(const char *name)
        Get information about cell-based arrays.  As is typical with
        readers this in only valid after the filename is set and
        update_information() has been called.
        """
        ret = self._wrap_call(self._vtk_obj.GetCellArrayStatus, *args)
        return ret

    def set_cell_array_status(self, *args):
        """
        V.set_cell_array_status(string, int)
        C++: void SetCellArrayStatus(const char *name, int status)
        Get information about cell-based arrays.  As is typical with
        readers this in only valid after the filename is set and
        update_information() has been called.
        """
        ret = self._wrap_call(self._vtk_obj.SetCellArrayStatus, *args)
        return ret

    def get_set_status(self, *args):
        """
        V.get_set_status(string) -> int
        C++: int GetSetStatus(const char *gridname)
        Get/Set information about sets. As is typical with readers this
        is valid only after the filename as been set and
        update_information() has been called. Note that sets with non-zero
        Ghost value are not treated as sets that the user can select
        using this API.
        """
        ret = self._wrap_call(self._vtk_obj.GetSetStatus, *args)
        return ret

    def set_set_status(self, *args):
        """
        V.set_set_status(string, int)
        C++: void SetSetStatus(const char *gridname, int status)
        Get/Set information about sets. As is typical with readers this
        is valid only after the filename as been set and
        update_information() has been called. Note that sets with non-zero
        Ghost value are not treated as sets that the user can select
        using this API.
        """
        ret = self._wrap_call(self._vtk_obj.SetSetStatus, *args)
        return ret

    stride = traits.Array(auto_set=False, enter_set=True, shape=(3,), dtype=int, value=(1, 1, 1), cols=3, help=\
        """
        
        """
    )

    def _stride_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetStride,
                        self.stride)

    def get_point_array_status(self, *args):
        """
        V.get_point_array_status(string) -> int
        C++: int GetPointArrayStatus(const char *name)
        Get/Set the point array status.
        """
        ret = self._wrap_call(self._vtk_obj.GetPointArrayStatus, *args)
        return ret

    def set_point_array_status(self, *args):
        """
        V.set_point_array_status(string, int)
        C++: void SetPointArrayStatus(const char *name, int status)
        Get/Set the point array status.
        """
        ret = self._wrap_call(self._vtk_obj.SetPointArrayStatus, *args)
        return ret

    def get_grid_status(self, *args):
        """
        V.get_grid_status(string) -> int
        C++: int GetGridStatus(const char *gridname)
        Get/Set information about grids. As is typical with readers this
        is valid only after the filename as been set and
        update_information() has been called.
        """
        ret = self._wrap_call(self._vtk_obj.GetGridStatus, *args)
        return ret

    def set_grid_status(self, *args):
        """
        V.set_grid_status(string, int)
        C++: void SetGridStatus(const char *gridname, int status)
        Get/Set information about grids. As is typical with readers this
        is valid only after the filename as been set and
        update_information() has been called.
        """
        ret = self._wrap_call(self._vtk_obj.SetGridStatus, *args)
        return ret

    def get_cell_array_name(self, *args):
        """
        V.get_cell_array_name(int) -> string
        C++: const char *GetCellArrayName(int index)
        Get information about cell-based arrays.  As is typical with
        readers this in only valid after the filename is set and
        update_information() has been called.
        """
        ret = self._wrap_call(self._vtk_obj.GetCellArrayName, *args)
        return ret

    def get_grid_name(self, *args):
        """
        V.get_grid_name(int) -> string
        C++: const char *GetGridName(int index)
        Get/Set information about grids. As is typical with readers this
        is valid only after the filename as been set and
        update_information() has been called.
        """
        ret = self._wrap_call(self._vtk_obj.GetGridName, *args)
        return ret

    def _get_number_of_cell_arrays(self):
        return self._vtk_obj.GetNumberOfCellArrays()
    number_of_cell_arrays = traits.Property(_get_number_of_cell_arrays, help=\
        """
        Get information about cell-based arrays.  As is typical with
        readers this in only valid after the filename is set and
        update_information() has been called.
        """
    )

    def _get_number_of_grids(self):
        return self._vtk_obj.GetNumberOfGrids()
    number_of_grids = traits.Property(_get_number_of_grids, help=\
        """
        Get/Set information about grids. As is typical with readers this
        is valid only after the filename as been set and
        update_information() has been called.
        """
    )

    def _get_number_of_point_arrays(self):
        return self._vtk_obj.GetNumberOfPointArrays()
    number_of_point_arrays = traits.Property(_get_number_of_point_arrays, help=\
        """
        Description: Returns the name for the active domain. Note that
        this may be different from what get_domain_name() returns if
        domain_name is NULL or invalid.
        
        Get information about point-based arrays. As is typical with
        readers this in only valid after the filename is set and
        update_information() has been called.
        """
    )

    def _get_number_of_set_arrays(self):
        return self._vtk_obj.GetNumberOfSetArrays()
    number_of_set_arrays = traits.Property(_get_number_of_set_arrays, help=\
        """
        These methods are provided to make it easier to use the Sets in
        para_view.
        """
    )

    def _get_number_of_sets(self):
        return self._vtk_obj.GetNumberOfSets()
    number_of_sets = traits.Property(_get_number_of_sets, help=\
        """
        Get/Set information about sets. As is typical with readers this
        is valid only after the filename as been set and
        update_information() has been called. Note that sets with non-zero
        Ghost value are not treated as sets that the user can select
        using this API.
        """
    )

    def get_point_array_name(self, *args):
        """
        V.get_point_array_name(int) -> string
        C++: const char *GetPointArrayName(int index)
        Returns the name of point array at the give index. Returns NULL
        if index is invalid.
        """
        ret = self._wrap_call(self._vtk_obj.GetPointArrayName, *args)
        return ret

    def _get_sil(self):
        return wrap_vtk(self._vtk_obj.GetSIL())
    sil = traits.Property(_get_sil, help=\
        """
        SIL describes organization of/relationships between
        classifications eg. blocks/materials/hierarchies.
        """
    )

    def _get_sil_update_stamp(self):
        return self._vtk_obj.GetSILUpdateStamp()
    sil_update_stamp = traits.Property(_get_sil_update_stamp, help=\
        """
        Every time the SIL is updated a this will return a different
        value.
        """
    )

    def get_set_array_name(self, *args):
        """
        V.get_set_array_name(int) -> string
        C++: const char *GetSetArrayName(int index)"""
        ret = self._wrap_call(self._vtk_obj.GetSetArrayName, *args)
        return ret

    def get_set_array_status(self, *args):
        """
        V.get_set_array_status(string) -> int
        C++: int GetSetArrayStatus(const char *name)"""
        ret = self._wrap_call(self._vtk_obj.GetSetArrayStatus, *args)
        return ret

    def get_set_name(self, *args):
        """
        V.get_set_name(int) -> string
        C++: const char *GetSetName(int index)
        Get/Set information about sets. As is typical with readers this
        is valid only after the filename as been set and
        update_information() has been called. Note that sets with non-zero
        Ghost value are not treated as sets that the user can select
        using this API.
        """
        ret = self._wrap_call(self._vtk_obj.GetSetName, *args)
        return ret

    def can_read_file(self, *args):
        """
        V.can_read_file(string) -> int
        C++: virtual int CanReadFile(const char *filename)
        Determine if the file can be read with this reader.
        """
        ret = self._wrap_call(self._vtk_obj.CanReadFile, *args)
        return ret

    _updateable_traits_ = \
    (('vectors_name', 'GetVectorsName'), ('tensors_name',
    'GetTensorsName'), ('normals_name', 'GetNormalsName'), ('file_name',
    'GetFileName'), ('scalars_name', 'GetScalarsName'),
    ('read_all_vectors', 'GetReadAllVectors'), ('stride', 'GetStride'),
    ('read_all_color_scalars', 'GetReadAllColorScalars'),
    ('read_all_fields', 'GetReadAllFields'), ('field_data_name',
    'GetFieldDataName'), ('reference_count', 'GetReferenceCount'),
    ('global_warning_display', 'GetGlobalWarningDisplay'),
    ('lookup_table_name', 'GetLookupTableName'), ('read_all_t_coords',
    'GetReadAllTCoords'), ('read_all_tensors', 'GetReadAllTensors'),
    ('read_from_input_string', 'GetReadFromInputString'), ('domain_name',
    'GetDomainName'), ('read_all_scalars', 'GetReadAllScalars'),
    ('read_all_normals', 'GetReadAllNormals'), ('abort_execute',
    'GetAbortExecute'), ('t_coords_name', 'GetTCoordsName'),
    ('release_data_flag', 'GetReleaseDataFlag'), ('debug', 'GetDebug'),
    ('progress', 'GetProgress'), ('input_string', 'GetInputString'),
    ('progress_text', 'GetProgressText'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['abort_execute', 'debug', 'global_warning_display',
    'read_all_color_scalars', 'read_all_fields', 'read_all_normals',
    'read_all_scalars', 'read_all_t_coords', 'read_all_tensors',
    'read_all_vectors', 'read_from_input_string', 'release_data_flag',
    'domain_name', 'field_data_name', 'file_name', 'input_string',
    'lookup_table_name', 'normals_name', 'progress_text', 'scalars_name',
    'stride', 't_coords_name', 'tensors_name', 'vectors_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(XdmfReader, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit XdmfReader properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View((['read_all_color_scalars', 'read_all_fields',
            'read_all_normals', 'read_all_scalars', 'read_all_t_coords',
            'read_all_tensors', 'read_all_vectors', 'read_from_input_string'], [],
            ['domain_name', 'field_data_name', 'file_name', 'input_string',
            'lookup_table_name', 'normals_name', 'scalars_name', 'stride',
            't_coords_name', 'tensors_name', 'vectors_name']),
            title='Edit XdmfReader properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit XdmfReader properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

